/*
 *  **************************************************************************
 *       May 2014- v0.1
 *       Ali Neissi
 *       Fabacademy 2014
 *  ************************************************************************** 
 *       This piece of code is for controlling the temperture of a 
 *       Soldering Iron which has also been built as the final project of 
 *       for Fabacademy 2014.  
 *  **************************************************************************    
 *       This project is using an ATmega328P, Thermocouple controller, lCD,
 *       Mosfet, transformer
 *  **************************************************************************
 *       The Iron is fed with 24 volts DC and it has 100 watts of power.
 *
 *  **************************************************************************
 */

// ---------------------------------------------------------------------------

/*
    ******Pinout**************************************************************
 
    ------buttons-------------------------------------------------------------
 *  PB1 (pin 13)-upButton  
 *  PB0 (pin 12)-downButton  
 *  ------Thermocouple---------------------------------------------------   
 *  PB4 (pin 16)-SO on MAX6675
 *  PB5 (pin 17)-SCK on MAX6675
 *  PB2 (pin 14)- ~CS on MAX6675
 *  ------Power Mosfet--------------------------------------------------------
 *  PD3 (pin 1)-RFD16N05LSM9A  // This is the PWM signal that is connected to 
 *                             // the source in the power Mosfet
 *  ------LCD-----------------------------------------------------------------
 *  PC0 (pin 23)-11 on LCD (DB4)
 *  PC1 (pin 24)-12 on LCD (DB5)
 *  PC2 (pin 25)-13 on LCD (DB6)
 *  PC3 (pin 26)-14 on LCD (DB7)
 *  (pin 32)-E  on LCD   // they cant be usedu
 *  (pin 9)-Rs on LCD  // they can't be used
 */    
//----------------------------------------------------------------------------

//  ******Libraries***********************************************************
    #include<LiquidCrystal.h>   // Adding the library for LCD
    #include<MAX6675.h>         // Adding the library for thermocouple IC
    #include <PID_v1.h>         // Adding the library for PID controller
//  **************************************************************************

const int upButton=2;   //The button for increasing the temperture is connected to pin number 2
const int downButton=4; //The button for decreasing the temperture is connected to pin number 4

boolean upPreviousButtonState=HIGH;  // Variable contaning the pervious button state
boolean upCurrentButtonState= HIGH; // Variable containg the current button state
boolean downPreviousButtonState=HIGH;
boolean downCurrentButtonState=HIGH;

int temperature; 

//********LCD*****************************************************************
LiquidCrystal lcd (5,2,A0,A1,A2,A3); //Initializing the library with the pins connected to the microcontroller. 
                             // Rs, E, DB4, DB5, DB6, DB7

byte degreeSymbol[8]= {
    B00110,
    B01001,
    B01001,
    B00110,
    B00000,
    B00000,
    B00000,
    B00000,
};


/*
 * Debounce function
 * This function gets the button and its previous state then returns the current button state
*/

boolean deboubce(boolean previous, int button);
{
    boolean current= digitalRead(button);
      if (previous !=current)
      {
        delay(5);
        current = digitalRead (button);
      }
    return current;
}

void setup() {
    
    pinMode (upButton, INPUT); //  Set the pin to which upButton is conencted as input
    pinMode (downButton, INPUT); //Set the pin to which downButton is connected as input
    pinMode  (A0, OUTPUT);
    pinMode (A1, OUTPUT);
    pinMode (A2, OUTPUT);
    pinMode (A3, OUTPUT);
    lcd.begin(16,2) ;
    
    // Making a custong character for the degree
    lcd.createChar(0, degree);
    // Printing the temperture on the LCD
    lcd.setCursor(0,0);
    lcd.print("Temp:"); 
    lcd.setCursor(8,0);
    lcd.write((byte)0);
    lcd.setCursor(9,0);
    lcd.print("C");
}

void loop() {
  // Debouncing the buttons
  upCurrentButtonState= debounce(upPreviousButtonState, upButton); 
  downCurrentButtonState= debounce(downPreviousButtonState, downButton);
  
  //Turn down the temprature
  if (downPreviousButtonState == HIGH && downCurrentButtonState == LOW)
  {
      temperature--;
  
  }

  // Turn up the temperture
  else if (upPreviousButtonState == HIGH && upCurrentButtonState == LOW)
  {
      temperature++;
  }
  
  // Print the temperture
  lcd.setCursor(5,0);
  lcd.print(temperature);
  
  downPreviousButtonState=downCurrentButtonState;
  upPreviousButtonState=upCurrentButtonState;
  
}
